/* File: code03e9.c
   ============================
 This file contains a patch for the trap "EvtGetEvent".  All it does it pass
control over to the standard FrmDrawForm function and then and time after the form title.

   Based on the GreenLightHack by Douglas I. Anderson (dianders@cs.stanford.edu)
   special thank to him.
   
   renaud Malaval (renaud.malaval2@fnac.net)

	File
	----
	code03e9.c

	Hack patch info
   ---------------
   code id:   0x1001
   atrap:	  0xA11D (sysTrapEvtGetEvent) 41245

   History
   -------
   15/06/00 Created
*/

// ----------------------------------
// Interfaces...
// ----------------------------------

/* System-wide */
#include <PalmTypes.h>
#include <SoundMgr.h>
#include <FeatureMgr.h>

#include <TimeMgr.h>
#include <Chars.h>

/* Hack specific */
#include "ResourceDefines.h"
#include "ClockEveryWhere.h"
#include "utils.h"

// ----------------------------------
// Local types...
// ----------------------------------

typedef void (*EvtGetEventFn) (EventPtr event, Int32 timeout);

// ----------------------------------
// Local function prototypes...
// ----------------------------------

// ----------------------------------
// Trap Handler...
// ----------------------------------

/* Function: MyEvtGetEvent()
   --------------------------
   This is the entry function of this patch.  All system calls to EvtGetEvent
   will be redirected here.  Basically, if the caller is actually using this
   function to display the form.
   
   @param frm      Tells the form to patch
   @return         nothing.
 */

// This is the code that will be called whenever ANY app calls EvtGetEvent,
// and CLockEveryWhere is enabled in HackMaster. HackMaster is not smart about the
// content of this file; it will simply jump to the first executable statement
// in this file. If you need helper functions, prototype them above, and
// define them AFTER the trap patch.
void	MyEvtGetEvent (EventPtr event, Int32 timeout)
{
	EvtGetEventFn	theOldTrap;
	UInt32 			theTemp;
	UInt32			theDisplayClockPeriod;
	CEWpref_t		thePrefs;


    // get the address of the old FrmDrawForm() from HackMaster
	if (FtrGet (kCreatorCode, kHackMasterTrapID_2, &theTemp))
	{
		SndPlaySystemSound (sndWarning);
	}
	else {
		UInt32		theNextTickValueToDisplayClock = 0;	/* pre affectation for 1st time (no feature) */
		UInt32		theTimeLeft = timeout;

		// set the function pointer to the old trap
		theOldTrap = (EvtGetEventFn)theTemp;
		// Since this hack is patching FrmDrawForm, there's not really much
		// useful work we can do BEFORE making the call to the OS, to get the
		// event, so we call the linked FrmDrawForm() first. This is not always
		// the case; sometimes you want to do something, then call the original
		// trap routine, then maybe do something else. This design decision is far
		// beyond the scope of this Sample.

		Utils_GetHackPref( &thePrefs);

		theDisplayClockPeriod = SysTicksPerSecond();
		if (thePrefs.BlinkingSecondControlvalue)
			theDisplayClockPeriod >>= 1;			// all 1/2 seconds
		else
			theDisplayClockPeriod *= 30;			// all 1/2 minutes

		FtrGet (kCreatorCode, kFeatureNumTicks, &theNextTickValueToDisplayClock);

		do
		{
			UInt32	theCurrentlyTick;
			Boolean	theForceReDraw = false;
			Boolean	theUpdate = false;
			
			theOldTrap (event, (theDisplayClockPeriod > theTimeLeft)?theTimeLeft:theDisplayClockPeriod);

			theCurrentlyTick = TimGetTicks();

			// use vchrHardPower to catch power off (sleep)
			// use vchrLateWakeup to catch power on (wake up)
			if ((event->eType == keyDownEvent) && (event->data.keyDown.chr == vchrLateWakeup))
			{
				theUpdate = true;
				theForceReDraw = true;
			}
			else if ((event->eType == nilEvent) && (theCurrentlyTick > theNextTickValueToDisplayClock))
			{	
				theUpdate = true;
//				theForceReDraw = false;
			}
			
			if (theUpdate)
			{
				FormType	*theActiveForm = FrmGetActiveForm ();

				if ( theActiveForm)
				{
					theNextTickValueToDisplayClock = theCurrentlyTick + theDisplayClockPeriod;
					FtrSet (kCreatorCode, kFeatureNumTicks, theNextTickValueToDisplayClock);
					Utils_DrawTheClock( theActiveForm, theForceReDraw, &thePrefs);
				}				
			}

			theTimeLeft -= theDisplayClockPeriod;

		} while (((Int32)theTimeLeft >= 0) && (event->eType == nilEvent));
	}
}	// MyEvtGetEvent

